''' <summary>
''' Provide methods to create PDF annotations.
''' </summary>
''' <remarks></remarks>
Public Class ClsPdfAnnotation
    Inherits clsPdfDictionnary
#Region "Private members"
#End Region
#Region "Constructors"
    ''' <summary>
    ''' Create a new annotation object
    ''' </summary>
    ''' <param name="type">Type of annotation to create</param>
    ''' <remarks>Default annotation location is (300, 300, 300, 300) </remarks>
    Sub New(ByVal type As AnnotationTypes)
        SubType = type
        Location(300, 300, 300, 300)    'Default location if none is specified
        Modified()
    End Sub
#End Region
#Region "Public Properties"
    ''' <summary>
    ''' Set the annotation's characteristics
    ''' </summary>
    ''' <value></value>
    ''' <remarks></remarks>
    Public WriteOnly Property SetCharacteristic() As AnnotationFlags
        Set(ByVal value As AnnotationFlags)
            MyBase.Add(PDF_F, CreateNumericObject(value))
        End Set
    End Property
    ''' <summary>
    ''' Get or Set the annotation's type
    ''' </summary>
    ''' <value>Annotation's type</value>
    ''' <returns>Annotation's type</returns>
    ''' <remarks></remarks>
    Public Property SubType() As AnnotationTypes
        Get
            SubType = _stringToAnnotationType(MyBase.GetDate(PDF_SUBTYPE))
        End Get
        Set(ByVal value As AnnotationTypes)
            Dim s As String = _annotationTypeToString(value)
            MyBase.Add(PDF_SUBTYPE, CreateNameObject(s), True)
        End Set
    End Property
    ''' <summary>
    ''' Get or set the annotation's content
    ''' </summary>
    ''' <value>Text to be put inside the annotation</value>
    ''' <returns>Text inside the annotation</returns>
    ''' <remarks></remarks>
    Public Property Contents() As String
        Get
            Contents = MyBase.GetString(PDF_CONTENTS)
        End Get
        Set(ByVal value As String)
            MyBase.Add(PDF_CONTENTS, CreateLiteralStringObject(value), True)
        End Set
    End Property
#End Region
#Region "Public Methods"
    ''' <summary>
    ''' Set the location of the annotation on a page.
    ''' </summary>
    ''' <param name="llx">Lower-left x position</param>
    ''' <param name="lly">Lower-left y position</param>
    ''' <param name="urx">Upper-right x position</param>
    ''' <param name="ury">Upper-right y position</param>
    ''' <remarks>All values are in user units. 
    ''' The annotations position is determined by the coordinates of the upper-left corner of its annotation rectangle and interpreted in the default user space of the page. 
    ''' When the default user space is scaled or rotated, the positions of the other three corners of the annotation location are different in the altered user space than they were in the original user space</remarks>
    Public Sub Location(ByVal llx As Double, ByVal lly As Double, ByVal urx As Double, ByVal ury As Double)
        Rectangle = CreateRectangle(llx, lly, urx, ury)
    End Sub
    ''' <summary>
    ''' Get annotation's characteristics
    ''' </summary>
    ''' <param name="flag">The annontation characteristic to test</param>
    ''' <returns>True if the annotation has the characteristic, False otherwise.</returns>
    ''' <remarks></remarks>
    Public Function HasCharacteristic(ByVal flag As AnnotationFlags) As Boolean
        Dim annotationFlags As Long
        Dim obj As clsPdfBaseObject
        obj = MyBase.Item(PDF_F)
        If Not obj Is Nothing Then
            annotationFlags = obj.GetLong
        Else
            annotationFlags = 0
        End If
        HasCharacteristic = ((flag And annotationFlags) = flag)
    End Function
#End Region
#Region "Friend Properties"
    Friend Property Rectangle() As PdfRectangle
        Get
            Rectangle = MyBase.GetRectangle(PDF_RECT)
        End Get
        Set(ByVal value As PdfRectangle)
            MyBase.Add(PDF_RECT, CreateRectangleObject(value), True)
        End Set
    End Property
#End Region
#Region "Private Methods"
    Private Function _annotationTypeToString(ByVal type As AnnotationTypes) As String
        Select Case SubType
            Case AnnotationTypes.Text
                _annotationTypeToString = PDF_TEXT
            Case Else
                _annotationTypeToString = ""
        End Select
    End Function
    Private Function _stringToAnnotationType(ByVal type As String) As AnnotationTypes
        Select Case type
            Case PDF_TEXT
                _stringToAnnotationType = AnnotationTypes.Text
            Case Else
                _stringToAnnotationType = AnnotationTypes.Text
        End Select
    End Function
#End Region
End Class
