''' <summary>
''' Provide access to the PDF document information properties
''' </summary>
''' <remarks></remarks>
Public Class ClsPdfDocumentInformation
    Inherits clsPdfDictionnary
#Region "Private members"
#End Region
#Region "Constructors"
    Friend Sub New(ByVal dict As Hashtable)
        MyBase.New(dict)
    End Sub
    Friend Sub New()
        MyBase.New()
    End Sub
#End Region
#Region "Public Properties"
    ''' <summary>
    ''' Get the modification date of the PDF file
    ''' </summary>
    ''' <value></value>
    ''' <returns>Modification date in human-readable format</returns>
    ''' <remarks></remarks>
    Public ReadOnly Property GetModificationDate() As String
        Get
            GetModificationDate = ModificationDate
            Modified()
        End Get
    End Property
    ''' <summary>
    ''' Set the modification date of the PDF file
    ''' </summary>
    ''' <value>Modification date in .NET Date format</value>
    ''' <remarks>The modification date is set automatically when the PDF is saved</remarks>
    Public WriteOnly Property SetModificationDate() As Date
        Set(ByVal value As Date)
            ModificationDate = CreatePdfDate(value)
            Modified()
        End Set
    End Property
    ''' <summary>
    ''' Get the modification date of the PDF file
    ''' </summary>
    ''' <value></value>
    ''' <returns>Creation date in human-readable format</returns>
    ''' <remarks></remarks>
    Public ReadOnly Property CreationDate() As String
        Get
            CreationDate = MyBase.GetDate(PDF_CREATIONDATE)
        End Get
    End Property
    ''' <summary>
    ''' Get or set the producer name
    ''' </summary>
    ''' <value>The PDF's producer name</value>
    ''' <returns>The PDF's producer name</returns>
    ''' <remarks>If the document was converted to PDF from another format, the name of the application (for example, Acrobat Distiller) that converted it to PDF.</remarks>
    Public Property Producer() As String
        Get
            Producer = MyBase.GetString(PDF_PRODUCER)
        End Get
        Set(ByVal value As String)
            MyBase.Add(PDF_PRODUCER, CreateLiteralStringObject(value), True)
            Modified()
        End Set
    End Property
    ''' <summary>
    ''' Get or set the creator name
    ''' </summary>
    ''' <value>The PDF's creator name</value>
    ''' <returns>The PDF's creator name</returns>
    ''' <remarks>If the document was converted to PDF from another format, the name of the application (for example, Adobe FrameMaker) that created the original document from which it was converted.</remarks>
    Public Property Creator() As String
        Get
            Creator = MyBase.GetString(PDF_CREATOR)
        End Get
        Set(ByVal value As String)
            MyBase.Add(PDF_CREATOR, CreateLiteralStringObject(value), True)
            Modified()
        End Set
    End Property
    ''' <summary>
    ''' Get or set the PDF keywords
    ''' </summary>
    ''' <value>The PDF's keywords</value>
    ''' <returns>The PDF's keywords</returns>
    ''' <remarks></remarks>
    Public Property Keywords() As String
        Get
            Keywords = MyBase.GetString(PDF_KEYWORDS)
        End Get
        Set(ByVal value As String)
            MyBase.Add(PDF_KEYWORDS, CreateLiteralStringObject(value), True)
            Modified()
        End Set
    End Property
    ''' <summary>
    ''' Get or set the subject
    ''' </summary>
    ''' <value>The PDF's subject</value>
    ''' <returns>The PDF's subject</returns>
    ''' <remarks></remarks>
    Public Property Subject() As String
        Get
            Subject = MyBase.GetString(PDF_SUBJECT)
        End Get
        Set(ByVal value As String)
            MyBase.Add(PDF_SUBJECT, CreateLiteralStringObject(value), True)
            Modified()
        End Set
    End Property
    ''' <summary>
    ''' Get or set the title
    ''' </summary>
    ''' <value>The PDF's title</value>
    ''' <returns>The PDF's title</returns>
    ''' <remarks></remarks>
    Public Property Title() As String
        Get
            Title = MyBase.GetString(PDF_TITLE)
        End Get
        Set(ByVal value As String)
            MyBase.Add(PDF_TITLE, CreateLiteralStringObject(value), True)
            Modified()
        End Set
    End Property
    ''' <summary>
    ''' Get or set the name of the person who created the document
    ''' </summary>
    ''' <value>The PDF's author name</value>
    ''' <returns>The PDF's author name</returns>
    ''' <remarks></remarks>
    Public Property Author() As String
        Get
            Author = MyBase.GetString(PDF_AUTHOR)
        End Get
        Set(ByVal value As String)
            MyBase.Add(PDF_AUTHOR, CreateLiteralStringObject(value), True)
            Modified()
        End Set
    End Property
#End Region
#Region "Friend Properties"
    Friend Property ModificationDate() As String
        Get
            ModificationDate = MyBase.GetDate(PDF_MODDATE)
        End Get
        Set(ByVal value As String)
            MyBase.Add(PDF_MODDATE, CreateLiteralStringObject(value), True)
        End Set
    End Property
#End Region
End Class
